/*@ ----------------------------------------------------------------------------

 
 
                        This file is part of the class library
 
            SoPlex  --- the Sequential object-oriented simPlex
 
            (c) by      Roland Wunderling
                        Konarad Zuse Zentrum fuer Informationstechnik Berlin
                        Heilbronner Str. 10, 
                        D-10711 Berlin, Germany
 
        There is absolutely no right to redistribute this file without the
        explicite permission by the authour.
 
 
    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 


   Class #LPRowSet#

   Identification:
   $Id: lprowset.hh,v 1.1 1998/07/16 12:11:29 boehm Exp $

   Program history:
   $Log: lprowset.hh,v $
   Revision 1.1  1998/07/16 12:11:29  boehm
   *** empty log message ***

// Revision 2.6  1996/01/08  12:23:17  bzfwunde
// Moved to new non-GNU generic Makefile environment
//
// Revision 2.5  1995/11/21  16:20:57  bzfwunde
// introduced SUBDIR_INCLUDE
//
// Revision 2.4  1995/10/13  15:29:17  bzfwunde
// minor improvements
//
// Revision 2.3  1995/06/15  13:13:48  bzfwunde
// minor changes (friend declaration)
//
// Revision 2.2  1995/05/17  11:38:27  bzfwunde
// bug fix? --- I hope :-)
//
// Revision 2.1  1995/03/31  14:59:57  bzfwunde
// tested Version running with set packing
//
// Revision 1.2  1995/03/03  18:50:12  bzfwunde
// changes to the interface
// minor bug fixes
//
// Revision 1.1.1.1  1995/01/12  17:35:32  bzfwunde
// initial, tested version
//

    ----------------------------------------------------------------------------
*/
#ifndef DEFlprowset		// prevent multiple includes
#define DEFlprowset


//@ ----------------------------------------------------------------------------
/*	\Section{Imports}
    Import required system include files
 */
#include <assert.h>


/*  and class header files
 */
#ifndef	SUBDIR_INCLUDE

#include "lprow.hh"
#include "dvector.hh"
#include "svset.hh"

#else 	// #SUBDIR_INCLUDE#

#include "lprow/lprow.hh"
#include "dvector/dvector.hh"
#include "svset/svset.hh"

#endif	// #SUBDIR_INCLUDE#



//@ ----------------------------------------------------------------------------
/* \Section{Class Declaration}
 */

/** set of LP rows. 
    Class #LPRowSet# implements a set of #LPRow#s. Unless for memory
    limitations, any number of #LPRow#s may be #add#ed to an #LPRowSet#. Single
    or multiple #LPRow#s may be #add#ed to an #LPRowSet#, where each method
    #add()# comes with two different signatures. One with an one without a
    parameter, used for returning the #Key#s assigned to the new #LPRow#s
    by the set. See \Ref{DataSet::Key} for a more detailed description of the
    concept of #Key#s. For the concept of renumbering #LPRow#s within an
    #LPRowSet# after removal of some #LPRow#s see \Ref{DataSet}.
 */
class LPRowSet : protected SVSet
{
private:
    DVector	left, right ;
    friend class	GlobalPointer ;

public:
	/// 
    typedef SVSet::Key	Key ;

    /**@name Inquiry */
    //@{
	/// number of #LPRow#s in #LPRowSet#. 
    int			num() const			{ return SVSet::num() ; }
	/// maximum number of #LPRow#s that fit. 
    int			max() const			{ return SVSet::max() ; }

	/// 
    const Vector&	lhs() const			{ return left ; }
	/// vector of #lhs# values. 
    Vector&		lhs()				{ return left ; }

	/// 
    double		lhs( int i ) const		{ return left[i] ; }
	/// #lhs# of #i#-th #LPRow#. 
    double&		lhs( int i )			{ return left[i] ; }

	/// 
    double		lhs( Key k ) const		{ return left[number(k)] ; }
	/// #lhs# of #k#-th #LPRow# in #LPRowSet#. 
    double&		lhs( Key k )			{ return left[number(k)] ; }

	/// 
    const Vector&	rhs() const			{ return right ; }
	/// vector of #rhs# values. 
    Vector&		rhs()				{ return right ; }

	/// 
    double		rhs( int i ) const		{ return right[i] ; }
	/// #rhs# of #i#-th #LPRow#. 
    double&		rhs( int i )			{ return right[i] ; }

	/// 
    double		rhs( Key k ) const		{ return right[number(k)] ; }
	/// #rhs# of #k#-th #LPRow#. 
    double&		rhs( Key k )			{ return right[number(k)] ; }

	/// 
    SVector&		rowVector( int i )		{ return operator[](i) ; }
	/// #rowVector# of #i#-th #LPRow#. 
    const SVector&	rowVector( int i ) const	{ return operator[](i) ; }

	/// 
    const SVector&	rowVector( Key k ) const	{ return operator[](k) ; }
	/// #rowVector# of #k#-th #LPRow#. 
    SVector&		rowVector( Key k )		{ return operator[](k) ; }

	/// 
    LPRow::Type		type( int i ) const
    			{
			    if( rhs(i) >= LPRow::infinity )
				return LPRow::GREATER_EQUAL ;
			    if( lhs(i) <= -LPRow::infinity )
				return LPRow::LESS_EQUAL ;
			    if( lhs(i) == rhs(i) )
				return LPRow::EQUAL ;
			    return LPRow::RANGE ;
			}
	/// inequality type of #k#-th #LPRow#. 
    LPRow::Type		type( Key k ) const		{ return type(number(k)) ; }

	/// 
    double		value( int i ) const
    			{
			    if( rhs(i) < LPRow::infinity )
				return rhs(i) ;
			    else
			    {
				assert( lhs(i) > -LPRow::infinity ) ;
				return lhs(i) ;
			    }
			}
	/// value of #k#-th #LPRow#. 
    double		value( Key k ) const		{ return value(number(k)) ; }

	/// return #Key# of #i#-th #LPRow# in #LPRowSet#. 
    Key		key( int i ) const			{ return SVSet::key(i) ; }
	/// return number of #k#-th #LPRow# in #LPRowSet#. 
    int		number( Key k ) const			{ return SVSet::number(k) ; }
	/// does #Key k# belong to #LPRowSet#?. 
    int		has( Key k ) const			{ return SVSet::has(k) ; }
    //@}


    /**@name Extension
	Extension methods come with two signatures, one of which providing a
	parameter to return the assigned #Key#(s). See \Ref{DataSet} for a more
	detailed description. All extension methods will automatically rearrange
	or allocate more memory if required.
     */
    //@{
	/// 
    void	add(const LPRow& row)
    		{
		    Key	k ;
		    add(k, row) ;
		}
	/// add #row# to #LPRowSet#. 
    void	add(Key& key, const LPRow& row)
    		{   add( key, row.lhs(), row.rowVector(), row.rhs() ) ;   }

	/// 
    void	add(double lhs, const SVector& rowVector, double rhs)
    		{
		    Key	k ;
		    add(k, lhs, rowVector, rhs) ;
		}
	/// add #LPRow# consisting of #lhs#, #rowVector# and #rhs# to #LPRowSet#. 
    void	add(Key& key, double lhs, const SVector& rowVector, double rhs) ;

	/// 
    void	add( const LPRowSet& set ) ;
	/// add all #LPRow#s of #set# to #LPRowSet#. 
    void	add(Key key[], const LPRowSet& set ) ;

	/// extend row #n# to fit #newmax# nonzeros. 
    void	xtend(int n, int newmax)
    		{   SVSet::xtend(rowVector(n), newmax ) ; }
	/// extend row #key# to fit #newmax# nonzeros. 
    void	xtend(Key key, int newmax)
    		{   SVSet::xtend(rowVector(key), newmax ) ; }
	/// 
    void	add2(Key k, int n, int idx[], double val[])
    		{   SVSet::add2(rowVector(k), n, idx, val) ; }
	/// add #n# nonzero (#idx#, #val#) to #i#-th #rowVector#.. 
    void	add2(int i, int n, int idx[], double val[])
    		{   SVSet::add2(rowVector(i), n, idx, val) ; }

	/// 
    SVector&	create(int nonzeros = 0, double lhs = 0, double rhs = 1)
    		{
		    Key k ;
		    return create( k, nonzeros, lhs, rhs ) ;
		}
	/** Create new #LPRow# with specified parameters and return a reference
	    to its row vector.
	 */
    SVector&	create(Key& nkey, int nonzeros = 0, double lhs = 0, double rhs = 1) ;
    //@}


    /**@name Shrinking
	See \Ref{DataSet} for a description of the renumbering of the remaining
	#LPRow#s in a #LPRowSet# after the call of a removal method.
     */
    //@{
	/// remove #i#-th #LPRow#. 
    void	remove(int i) ;
	/// remove #k#-th #LPRow#. 
    void	remove(Key k)
    		{   remove( number(k) ) ;   }


	/// remove multiple elements. 
    void	remove(int perm[]) ;

	/// 
    void	remove(Key keys[], int n)
    		{
		    DataArray<int>	perm(num()) ;
		    remove( keys, n, perm ) ;
		}

	/// 
    void	remove(int nums[], int n)
    		{
		    DataArray<int>	perm(num()) ;
		    remove( nums, n, perm ) ;
		}

	/// 
    void	remove(Key keys[], int n, int* perm) ;

	/// remove #n# #LPRow#s. 
    void	remove(int nums[], int n, int* perm) ;

	/// remove all #LPRow#s. 
    void	clear() ;
    //@}


    /**@name Memory Management
	For a description of the memory management methods, see the
	documentation of #SVSet#, which has benn used for implementating
	#LPRowSet#.
     */
    //@{
	/// 
    void	reMax(int newmax = 0)
    		{
		    SVSet::reMax( newmax ) ;
		    left.reSize ( max() ) ;
		    right.reSize( max() ) ;
		}
	/// used nonzero memory. 
    int		memSize() const			{ return SVSet::memSize() ; }

	/// length of nonzero memory. 
    int		memMax() const			{ return SVSet::memMax() ; }

	/// reset length of nonzero memory. 
    void	memRemax(int newmax)	 	{ SVSet::memRemax(newmax) ; }

	/// garbage collection in nonzero memory. 
    void	memPack() 			{ SVSet::memPack() ; }
    //@}


    /**@name Miscellaneous */
    //@{
	    /// 
	LPRowSet&	operator=( const LPRowSet& rs )
	{
	    SVSet::operator=(rs) ;
	    left  = rs.left ;
	    right = rs.right ;
	    return *this ;
	}

	/// 
    		LPRowSet( int max = -1, int memmax = -1 )
		    : SVSet(max, memmax), left(0), right(0)		{ }

	/// check consistency. 
    int		isConsistent() const ;
    //@}
} ;


#endif // #DEFlprowset#
